'use strict';

var lodash = require('lodash');
var isPromise = require('is-promise');
var defaultStorage = require('./file-sync');

function low(source) {
  var _ref = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var _ref$storage = _ref.storage;
  var storage = _ref$storage === undefined ? defaultStorage : _ref$storage;
  var _ref$format = _ref.format;
  var format = _ref$format === undefined ? null : _ref$format;
  var _ref$writeOnChange = _ref.writeOnChange;
  var writeOnChange = _ref$writeOnChange === undefined ? true : _ref$writeOnChange;

  // Create a fresh copy of lodash
  var _ = lodash.runInContext();

  var db = _.chain({});

  if (source) {
    if (storage) {
      if (storage.read) {
        db.read = function () {
          var s = arguments.length <= 0 || arguments[0] === undefined ? source : arguments[0];

          var res = storage.read(s, db.deserialize);
          var init = function init(obj) {
            db.__wrapped__ = obj;
            db._checksum = JSON.stringify(db.__wrapped__);
          };

          if (isPromise(res)) {
            return res.then(function (obj) {
              init(obj);
              return db;
            });
          }

          init(res);
          return db;
        };
      }

      if (storage.write) {
        db.write = function () {
          var dest = arguments.length <= 0 || arguments[0] === undefined ? source : arguments[0];
          return storage.write(dest, db.__wrapped__, db.serialize);
        };
      }
    }

    if (format) {
      var _options = options;
      var _format = _options.format;

      db.serialize = _format.serialize;
      db.deserialize = _format.deserialize;
    }
  }

  // Persist database state
  function persist() {
    if (db.source && db.write && writeOnChange) {
      var str = JSON.stringify(db.__wrapped__);

      if (str !== db._checksum) {
        db._checksum = str;
        db.write(db.source, db.__wrapped__);
      }
    }
  }

  // Modify value function to call save before returning result
  _.prototype.value = _.wrap(_.prototype.value, function (value) {
    var v = value.apply(this);
    persist();
    return v;
  });

  // Get or set database state
  db.getState = function () {
    return db.__wrapped__;
  };
  db.setState = function (state) {
    db.__wrapped__ = state;
    persist();
  };

  db._ = _;
  db.source = source;

  // Read
  if (db.read) {
    return db.read();
  } else {
    return db;
  }
}

module.exports = low;